import '../view/stage_items/stage_entity.dart';

mixin Selectable<Entity extends StageEntity> on StageEntityState<Entity> {
  bool selected = false;
  void Function() onSelect;

  /// basic behavior; override this in classes which are using the Selectable mixin
  void onSelected() {
    setState(() => selected = true);
  }

  void onUnselected() {
    setState(() => selected = false);
  }

  void select() {
    if (!mounted) return;
    onSelected();
  }

  void unselect() {
    if (!mounted) return;
    onUnselected();
  }

  void toggleSelected() {
    if (!mounted) return;
    if (selected) {
      onUnselected();
    } else {
      onSelected();
    }
  }
}
