import 'graph.dart';
import 'graph_node.dart';
import 'stage_theme.dart';
import 'package:flutter/material.dart';

class StageConfig {
  /// Initial list of nodes.
  final List<GraphNode> nodes;

  /// This function will be called, when the user press the the save button. Gives back the current state (represented by a graph) as a parameter.
  final void Function(Graph graph) onSave;

  /// This will be called when the user want to add a new node. The tapped position is passed as a parameter.
  final GraphNode Function(Offset position) onAddNew;

  /// Size of the stage area where the entities (nodes) exist and can be moved
  final Size stageSize;

  /// The center position of the camera.
  Offset
      initialCameraPosition; // should be multiplied by -1 (check the building of GestureTransformable in the stage.dart).

  /// Initial zoom value; smaller value = zoom out, bigger value = zoom in.
  final double initialZoom;

  /// Configure basic stage theme colors.
  final StageTheme theme;

  StageConfig(
      {this.nodes,
      this.onSave,
      this.onAddNew,
      this.stageSize = const Size(2000, 2000),
      this.initialZoom = 1.0,
      this.theme = const StageTheme(),
      Offset cameraPosition}) {
    this.initialCameraPosition =
        cameraPosition ?? this.stageSize.center(Offset(0, 0));
  }
}
