import 'package:flutter/material.dart';
import 'package:relations/bloc/bloc.dart';
import 'math_util.dart';
import 'dart:math' as math;
import '../models/graph_node.dart';

enum NodeArrange { none, circle, vertical, horizontal, matrix }

class NodeArrangement {
  static List<GraphNode> horizontal(Offset center, List<GraphNode> nodes,
      {int padding = 15}) {
    final int countNodes = nodes.length;
    final double largestNodeWidth =
        nodes.map((GraphNode n) => n.size.width).reduce(math.max);

    int rows;
    int columns;

    double amountOverlap =
        ((largestNodeWidth * countNodes) + ((countNodes - 1) * padding)) /
            StageBloc.stageBounds.width;
    if (amountOverlap <= 1.0) {
      /// can fit in one row
      rows = 1;
      columns = countNodes;
    } else {
      rows = amountOverlap.floor();
      columns = StageBloc.stageBounds.width ~/ (largestNodeWidth + padding);
    }

    _matrixFormation(center, nodes, rows, columns,
        padding: padding, greatestNodeWidth: largestNodeWidth);

    return nodes;
  }

  static List<GraphNode> vertical(Offset center, List<GraphNode> nodes,
      {int padding = 15}) {
    final int countNodes = nodes.length;
    final double largestNodeHeight =
        nodes.map((GraphNode n) => n.size.height).reduce(math.max);

    int rows;
    int columns;

    double amountOverlap =
        ((largestNodeHeight * countNodes) + ((countNodes - 1) * padding)) /
            StageBloc.stageBounds.height;

    if (amountOverlap <= 1.0) {
      /// can fit in one column
      columns = 1;
      rows = countNodes;
    } else {
      columns = (amountOverlap + 1.0).toInt();

      ///.floor(); not works correctly
      /// print("cols: $columns");
      rows = StageBloc.stageBounds.height ~/ (largestNodeHeight + padding);
    }

    _matrixFormation(center, nodes, rows, columns,
        padding: padding, greatestNodeHeight: largestNodeHeight);

    return nodes;
  }

  static void _matrixFormation(
      Offset center, List<GraphNode> nodes, int rows, int columns,
      {int padding = 15, double greatestNodeWidth, double greatestNodeHeight}) {
    final double largestNodeWidth = greatestNodeWidth ??
        nodes.map((GraphNode n) => n.size.width).reduce(math.max);
    final double largestNodeHeight = greatestNodeHeight ??
        nodes.map((GraphNode n) => n.size.height).reduce(math.max);

    final Offset topLeft = Offset(
        center.dx -
            ((largestNodeWidth * columns) + (columns - 1) * padding) / 2.0,
        center.dy - ((largestNodeHeight * rows) + rows * padding) / 2.0);

    for (int i = 0; i < nodes.length; i++) {
      GraphNode node = nodes[i];
      int y = i ~/ columns;
      int x = i % columns;
      double posX = topLeft.dx + (x * (largestNodeWidth + padding)).toDouble();
      double posY = topLeft.dy + (y * (largestNodeHeight + padding)).toDouble();
      node.position = Offset(posX, posY);
    }
  }

  static List<GraphNode> matrix(Offset center, List<GraphNode> nodes,
      {int padding = 15}) {
    int sqrt = math.sqrt(nodes.length).toInt();
    int rows = sqrt + 1;
    int columns = sqrt;

    /*
    print("countNodes: $countNodes");
    print("SQRT of countNodes: $sqrt");
    print("rows: $rows");
    print("columns: $columns");
    print("--------------------------------------------");
    */

    _matrixFormation(center, nodes, rows, columns, padding: padding);

    return nodes;
  }

  static List<GraphNode> circle(Offset center, List<GraphNode> nodes) {
    List<GraphNode> arranged = nodes;
    final int countNodes = arranged.length;
    final double angleSubdivision = MathUtil.twoPi / countNodes;
    final double radius = 300;

    /// may have another calculation method for this based on sizes of nodes
    double angle = -MathUtil.halfPi;

    for (var node in arranged) {
      Offset newCenter = Offset(center.dx + radius * math.cos(angle),
          center.dy + radius * math.sin(angle));
      node.locateByCenter(newCenter);
      angle += angleSubdivision;
    }
    return arranged;
  }
}
